#|
Arxiu: derivades.lisp
Contingut: funcions per al càlcul de derivades.
Autor: Felip Alàez Nadal.
Última actualització: 25-II-07

|#

#|BSD License

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. |#


(provide :ph_derivatives)
(defpackage :ph_derivatives
  (:use :cl)
  (:export #:primera_derivada #:segona_derivada #:derivada_creuada))

(in-package :ph_derivatives)

(defun primera_derivada (funcio punt variable &key (h 1e-2))
  "Calcula la primera derivada de funcio a punt, usant l'increment h donat. Punt ha de ser una llista, variable indica el nombre de variable, amb zero per a la primera, etc..."
  (let ((arguments (copy-seq punt))
	(arguments2 (copy-seq  punt)))
    (incf (nth variable arguments) h)
    (decf (nth variable arguments2 ) h)
    (/ (-  (apply funcio arguments) (apply funcio arguments2))(* 2 h))))


(defun segona_derivada (funcio punt variable &key (h 1e-2))
  "Computes the derivative of funcio at punt, using the variable. 0 for the first variable, 1 for the second..."
  (let (
	(arguments (copy-seq punt))
	(arguments2 (copy-seq punt)))
    (incf (nth variable arguments) h)
    (decf (nth variable arguments2 ) h)
    (/ (- (+ (apply funcio arguments) (apply funcio arguments2)) (* 2 (apply funcio punt))) (expt h 2))))


(defun derivada_creuada (funcio punt variable1 variable2 &key (h 1e-2))
  "Computes the derivative respect variable1 and variable2. Punt must be a list, variable1 and variable2 the number of variable, with 0 for the first..."
  (let
      ((arguments (copy-seq punt))
       primeres_derivades segones_derivades
       auxiliar auxiliar_der auxiliar_2der)
    (setq arguments (mapcar #'(lambda (x) (incf x h)) (list (nth variable1 arguments) (nth variable2 arguments) )))
    (setq arguments (concatenate 'list arguments (rest (rest punt))))
    (setq primeres_derivades (mapcar #'primera_derivada (list funcio funcio ) (list punt punt) (list variable1 variable2) '(:h :h) (list h h)))
    (setq segones_derivades (mapcar #'segona_derivada (list funcio funcio ) (list punt punt) (list variable1 variable2) '(:h :h) (list h h)))
    (setq auxiliar (- (apply funcio arguments) (apply funcio punt)))
    (setq auxiliar_der (* h (apply #'+ primeres_derivades)))
    (setq auxiliar_2der (/ (* (expt h 2) (apply #'+ segones_derivades)) 2))
    (/ (- auxiliar auxiliar_der auxiliar_2der) (expt h 2))))
    
		 
