#|
Arxiu: ph_montecarlo.lisp
Contingut: implementacions de métodes d'interpolació
Autor: Felip Alàez Nadal.
Útima actualització: 9-VI-06

|#

#|BSD License

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. |#



 (defpackage :ph_interpolation
   (:use :cl)
   (:export #:direct_interpolation )
   (:documentation "This package provides methods for interpolation."))

(in-package :ph_interpolation)

(defun direct_interpolation (  points)
  "Applies the formula P_{n} (x) =  sum_{i = 0 } ^{n } product _{k not i} (x - x_{k}) over  (x_{i} - x_{k}) . points is a list like ( (x1 y1) (x2 y2) ... (xn yn)). This is the simplest formula for an interpolation. But this is an unefficient method, due to: a) you must recompute the coefficients every time you evaluate the polynomial and b) you can't recalculate a higger degree polynomial with the previous one. But if you want only a simple interpolation formula, this is for you."
  (let (
	(len (length points)))
	(if (= len 0) (error "You must provide some points."))
	)
    (lambda (x)
      (let ((len (length points))
	    (suma 0)
	    (producte 1)
	    )

	(do ((i 0 (incf i)))
	    ((= i len ) suma)
	  (setq producte 1)
	  (do ((k 0 (incf k)))
	      ((>= k len ) nil)
	    (cond
	      ((= k i) 
	       nil)
	      (t 
	       (setq producte (* producte (/ (- x (first (nth k points))) (- (first (nth i points)) (first (nth k points)))))))))
	  (setq  suma (+ suma (* (second (nth i points))  producte)))
	  ))))
  